#!/usr/bin/env perl
#line 15
#
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#             G-language Image Viewer
#
#     Copyright (C) 2001 Keio University
#:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#
#   $Id: gimv.pl,v 1.1.1.1 2002/04/02 20:25:47 gaou Exp $
#
# G-language System is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# G-language System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with G-language System -- see the file COPYING.
# If not, write to the Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
#END_HEADER
#
# written by Kazuharu Arakawa <gaou@g-language.org> at
# G-language Project, Institute for Advanced Biosciences, Keio University.
#
use strict;
use Wx;


use vars qw($VERSION @ISA @EXPORT @EXPORT_OK @INC $AUTOLOAD);
use vars qw($filename $file $bmp);

my $convert = 'convert';
$convert = "start /B /ImageMagick/convert" if ('MSWin32' eq $^O);

$filename = shift;
$file = '';

if (length $filename){
	if ($filename =~ /.bmp$/){
		$file = $filename;
        }else{
		#my $dirfile = (split(/\//, $filename))[-1];
		$file = $filename . "\.bmp";
		system("$convert $filename $file");
	}
}else{
	print "==== gimv G-language Manager Image Viewer ====\n\n";
	print "  Copyright (C) 2001 G-language Project, Keio University\n";
	print "  usage: gimv *.png\n";
	print "  supports most graphic formats.\n\n";
	exit;
}

# hack for wxMotif compatibility
if( $Wx::_platform == $Wx::_motif ) { 
  eval 'sub Wx::Window::SetToolTip {}';
  eval 'sub Wx::ToolTip::new {}';
};


package MyApp;
use strict;
use vars qw(@ISA);
@ISA = qw(Wx::App);

use Wx qw(wxMENU_TEAROFF);

use Wx qw( wxBITMAP_TYPE_BMP );


sub OnInit {
    my( $this ) = @_;

    $main::bmp = Wx::Bitmap->new( $main::file, wxBITMAP_TYPE_BMP );
    $this->{FRAME} = MyFrame->new( undef, -1, "gimv " . $main::filename,
                                 [-1,-1], 
				[$main::bmp->GetWidth + 3, 
				 $main::bmp->GetHeight + 20]);

    $this->{FRAME}->Show( 1 );
    $this->SetTopWindow( $this->{FRAME} );

    1;
}


package MyFrame;

use strict;
use vars qw(@ISA);

use Wx qw(wxTheApp wxMENU_TEAROFF);

use Wx qw(wxTE_MULTILINE wxLB_SORT wxLB_ALWAYS_SB wxCB_SORT
 wxBLUE wxCROSS_CURSOR wxCURSOR_HAND wxPROCESS_ENTER wxTE_PROCESS_ENTER
 wxBITMAP_TYPE_XPM wxBITMAP_TYPE_BMP wxWHITE
 wxNOT_FOUND wxITALIC_FONT wxDefaultSize wxRA_SPECIFY_COLS wxRA_SPECIFY_ROWS
 wxRB_GROUP);
use Wx::Event qw(EVT_SIZE EVT_TEXT);

@ISA = qw(Wx::Panel);
@ISA = qw(Wx::Frame);

sub new {
    my $class = shift;
    my( $this ) = $class->SUPER::new( @_ );

    my $panel = Wx::Panel->new( $this, -1);

    use Wx qw(wxTheApp wxNullIcon wxNullBitmap);

    if( $Wx::_platform != $Wx::_motif ) {
      if( $main::bmp->Ok() ) {
        $main::bmp->SetMask( Wx::Mask->new( $main::bmp, wxBLUE) );
        $this->{bmp} = Wx::StaticBitmap->new( $panel, -1, $main::bmp, [0, 0] );
      }
    }


    $this->SetIcon( Wx::GetWxPerlIcon() );

    my( $file_menu ) = Wx::Menu->new( '', wxMENU_TEAROFF );

    $file_menu->Append( 1024, "Save &As...\tCtrl-S" );
    $file_menu->AppendSeparator();
    $file_menu->Append( 4, "E&xit\tCtrl-Q", 'Quit gimv' );

    my( $menu_bar ) = Wx::MenuBar->new();
    $menu_bar->Append( $file_menu, '&File' );

    $this->SetMenuBar( $menu_bar );

    use Wx::Event qw(EVT_CLOSE EVT_MENU);

    EVT_MENU( $this, 1024, \&SaveFile );
    EVT_MENU( $this, 4, \&OnQuit );
    EVT_CLOSE( $this, \&OnClose );

    $this;
}

sub OnSize {
    my( $this, $x, $y ) = @_;

    $this->{FRAME}->SetSize( 50, 50, $x, $y);
}

use Wx qw(wxSAVE wxOVERWRITE_PROMPT);

sub SaveFile {
  my $this = shift;

  my $last;

  if ('MSWin32' eq $^O){
	my @path = split(/\\/, $main::filename);
	$last = pop @path;
  }else{
	my @path = split(/\//, $main::filename);
        $last = pop @path;
  }

  my( $str ) = Wx::FileSelector( 'Save As...', undef, $last, undef, 'All file|*|BMP files (*.bmp)|*.bmp|GIF files (*.gif)|*.gif|JPEG files (*.jpg)|*.jpg|PBM files (*.pbm)|*.pbm|PNG files (*.png)|*.png|PPM files (*.ppm)|*.ppm|PS files (*.ps)|*.ps|TIF files (*.tif)|*.tif|',
wxSAVE);

  &OnSaveFile($this, $str) if (length $str);
}

sub OnSaveFile {
    my( $this, $str ) = @_;

    my $path_a = $main::file;
    $path_a =~ s/\//\\/g if ('MSWin32' eq $^O);
    my $path_b = $str;
    system("$convert $path_a \"$path_b\"");
}

sub OnQuit {
  my $this = shift;

  $this->Close( 1 );
}


sub OnClose {
  my( $this ) = shift;

  $this->Destroy;

  $_[0]->Skip();
}

package main;

my( $app ) = MyApp->new();

$app->MainLoop();	



1;
